"""
Copyright (C) 2025 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi.object_implem import Object
from ibapi.const import UNSET_DOUBLE, UNSET_DECIMAL
from ibapi.utils import decimalMaxString
from ibapi.utils import floatMaxString

class OrderAllocation(Object):
    def __init__(self):
        self.account = ""
        self.position = UNSET_DECIMAL
        self.positionDesired = UNSET_DECIMAL
        self.positionAfter = UNSET_DECIMAL
        self.desiredAllocQty = UNSET_DECIMAL
        self.allowedAllocQty = UNSET_DECIMAL
        self.isMonetary = False

    def __str__(self):
        s = ("Account: %s, Position: %s, PositionDesired: %s, PositionAfter: %s, "
             "DesiredAllocQty: %s, AllowedAllocQty: %s, IsMonetary: %s") % (
                str(self.account),
                decimalMaxString(self.position),
                decimalMaxString(self.positionDesired),
                decimalMaxString(self.positionAfter),
                decimalMaxString(self.desiredAllocQty),
                decimalMaxString(self.allowedAllocQty),
                str(self.isMonetary),
            )
        return s

class OrderState:
    def __init__(self):
        self.status = ""

        self.initMarginBefore = ""
        self.maintMarginBefore = ""
        self.equityWithLoanBefore = ""
        self.initMarginChange = ""
        self.maintMarginChange = ""
        self.equityWithLoanChange = ""
        self.initMarginAfter = ""
        self.maintMarginAfter = ""
        self.equityWithLoanAfter = ""

        self.commissionAndFees = UNSET_DOUBLE  # type: float
        self.minCommissionAndFees = UNSET_DOUBLE  # type: float
        self.maxCommissionAndFees = UNSET_DOUBLE  # type: float
        self.commissionAndFeesCurrency = ""
        self.marginCurrency = ""
        self.initMarginBeforeOutsideRTH = UNSET_DOUBLE  # type: float
        self.maintMarginBeforeOutsideRTH = UNSET_DOUBLE  # type: float
        self.equityWithLoanBeforeOutsideRTH = UNSET_DOUBLE  # type: float
        self.initMarginChangeOutsideRTH = UNSET_DOUBLE  # type: float
        self.maintMarginChangeOutsideRTH = UNSET_DOUBLE  # type: float
        self.equityWithLoanChangeOutsideRTH = UNSET_DOUBLE  # type: float
        self.initMarginAfterOutsideRTH = UNSET_DOUBLE  # type: float
        self.maintMarginAfterOutsideRTH = UNSET_DOUBLE  # type: float
        self.equityWithLoanAfterOutsideRTH = UNSET_DOUBLE  # type: float
        self.suggestedSize = UNSET_DECIMAL
        self.rejectReason = ""
        self.orderAllocations = None
        self.warningText = ""
        self.completedTime = ""
        self.completedStatus = ""
        
    def __str__(self):
        s = ("Status: %s, InitMarginBefore: %s, MaintMarginBefore: %s, EquityWithLoanBefore: %s, "
             "InitMarginChange: %s, MaintMarginChange: %s, EquityWithLoanChange: %s, "
             "InitMarginAfter: %s, MaintMarginAfter: %s, EquityWithLoanAfter: %s, "
             "CommissionAndFees: %s, MinCommissionAndFees: %s, MaxCommissionAndFees: %s, CommissionAndFeesCurrency: %s, MarginCurrency: %s, "
             "InitMarginBeforeOutsideRTH: %s, MaintMarginBeforeOutsideRTH: %s, EquityWithLoanBeforeOutsideRTH: %s, "
             "InitMarginChangeOutsideRTH: %s, MaintMarginChangeOutsideRTH: %s, equityWithLoanChangeOutsideRTH: %s, "
             "InitMarginAfterOutsideRTH: %s, MaintMarginAfterOutsideRTH: %s, equityWithLoanAfterOutsideRTH: %s, "
             "SuggestedSize: %s, RejectReason: %s, WarningText: %s, CompletedTime: %s, CompletedStatus: %s") % (
                str(self.status),
                str(self.initMarginBefore),
                str(self.maintMarginBefore),
                str(self.equityWithLoanBefore),
                str(self.initMarginChange),
                str(self.maintMarginChange),
                str(self.equityWithLoanChange),
                str(self.initMarginAfter),
                str(self.maintMarginAfter),
                str(self.equityWithLoanAfter),
                floatMaxString(self.commissionAndFees),
                floatMaxString(self.minCommissionAndFees),
                floatMaxString(self.maxCommissionAndFees),
                str(self.commissionAndFeesCurrency),
                str(self.marginCurrency),
                floatMaxString(self.initMarginBeforeOutsideRTH),
                floatMaxString(self.maintMarginBeforeOutsideRTH),
                floatMaxString(self.equityWithLoanBeforeOutsideRTH),
                floatMaxString(self.initMarginChangeOutsideRTH),
                floatMaxString(self.maintMarginChangeOutsideRTH),
                floatMaxString(self.equityWithLoanChangeOutsideRTH),
                floatMaxString(self.initMarginAfterOutsideRTH),
                floatMaxString(self.maintMarginAfterOutsideRTH),
                floatMaxString(self.equityWithLoanAfterOutsideRTH),
                decimalMaxString(self.suggestedSize),
                str(self.rejectReason),
                str(self.warningText),
                str(self.completedTime),
                str(self.completedStatus),
            )
        

        if self.orderAllocations:
            s += " OrderAllocations("
            for orderAllocation in self.orderAllocations:
                s += str(orderAllocation) + "; "
            s += ")"

        return s
        



"""
Copyright (C) 2023 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

""" Package implementing the Python API for the TWS/IB Gateway """

VERSION = {"major": 10, "minor": 37, "micro": 2}


def get_version_string():
    version = "{major}.{minor}.{micro}".format(**VERSION)
    return version


__version__ = get_version_string()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: OrderState.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'OrderState.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ibapi.protobuf.OrderAllocation_pb2 as OrderAllocation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10OrderState.proto\x12\x08protobuf\x1a\x15OrderAllocation.proto\"\xf5\r\n\nOrderState\x12\x13\n\x06status\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1d\n\x10initMarginBefore\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x1e\n\x11maintMarginBefore\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12!\n\x14\x65quityWithLoanBefore\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12\x1d\n\x10initMarginChange\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12\x1e\n\x11maintMarginChange\x18\x06 \x01(\x01H\x05\x88\x01\x01\x12!\n\x14\x65quityWithLoanChange\x18\x07 \x01(\x01H\x06\x88\x01\x01\x12\x1c\n\x0finitMarginAfter\x18\x08 \x01(\x01H\x07\x88\x01\x01\x12\x1d\n\x10maintMarginAfter\x18\t \x01(\x01H\x08\x88\x01\x01\x12 \n\x13\x65quityWithLoanAfter\x18\n \x01(\x01H\t\x88\x01\x01\x12\x1e\n\x11\x63ommissionAndFees\x18\x0b \x01(\x01H\n\x88\x01\x01\x12!\n\x14minCommissionAndFees\x18\x0c \x01(\x01H\x0b\x88\x01\x01\x12!\n\x14maxCommissionAndFees\x18\r \x01(\x01H\x0c\x88\x01\x01\x12&\n\x19\x63ommissionAndFeesCurrency\x18\x0e \x01(\tH\r\x88\x01\x01\x12\x1b\n\x0emarginCurrency\x18\x0f \x01(\tH\x0e\x88\x01\x01\x12\'\n\x1ainitMarginBeforeOutsideRTH\x18\x10 \x01(\x01H\x0f\x88\x01\x01\x12(\n\x1bmaintMarginBeforeOutsideRTH\x18\x11 \x01(\x01H\x10\x88\x01\x01\x12+\n\x1e\x65quityWithLoanBeforeOutsideRTH\x18\x12 \x01(\x01H\x11\x88\x01\x01\x12\'\n\x1ainitMarginChangeOutsideRTH\x18\x13 \x01(\x01H\x12\x88\x01\x01\x12(\n\x1bmaintMarginChangeOutsideRTH\x18\x14 \x01(\x01H\x13\x88\x01\x01\x12+\n\x1e\x65quityWithLoanChangeOutsideRTH\x18\x15 \x01(\x01H\x14\x88\x01\x01\x12&\n\x19initMarginAfterOutsideRTH\x18\x16 \x01(\x01H\x15\x88\x01\x01\x12\'\n\x1amaintMarginAfterOutsideRTH\x18\x17 \x01(\x01H\x16\x88\x01\x01\x12*\n\x1d\x65quityWithLoanAfterOutsideRTH\x18\x18 \x01(\x01H\x17\x88\x01\x01\x12\x1a\n\rsuggestedSize\x18\x19 \x01(\tH\x18\x88\x01\x01\x12\x19\n\x0crejectReason\x18\x1a \x01(\tH\x19\x88\x01\x01\x12\x33\n\x10orderAllocations\x18\x1b \x03(\x0b\x32\x19.protobuf.OrderAllocation\x12\x18\n\x0bwarningText\x18\x1c \x01(\tH\x1a\x88\x01\x01\x12\x1a\n\rcompletedTime\x18\x1d \x01(\tH\x1b\x88\x01\x01\x12\x1c\n\x0f\x63ompletedStatus\x18\x1e \x01(\tH\x1c\x88\x01\x01\x42\t\n\x07_statusB\x13\n\x11_initMarginBeforeB\x14\n\x12_maintMarginBeforeB\x17\n\x15_equityWithLoanBeforeB\x13\n\x11_initMarginChangeB\x14\n\x12_maintMarginChangeB\x17\n\x15_equityWithLoanChangeB\x12\n\x10_initMarginAfterB\x13\n\x11_maintMarginAfterB\x16\n\x14_equityWithLoanAfterB\x14\n\x12_commissionAndFeesB\x17\n\x15_minCommissionAndFeesB\x17\n\x15_maxCommissionAndFeesB\x1c\n\x1a_commissionAndFeesCurrencyB\x11\n\x0f_marginCurrencyB\x1d\n\x1b_initMarginBeforeOutsideRTHB\x1e\n\x1c_maintMarginBeforeOutsideRTHB!\n\x1f_equityWithLoanBeforeOutsideRTHB\x1d\n\x1b_initMarginChangeOutsideRTHB\x1e\n\x1c_maintMarginChangeOutsideRTHB!\n\x1f_equityWithLoanChangeOutsideRTHB\x1c\n\x1a_initMarginAfterOutsideRTHB\x1d\n\x1b_maintMarginAfterOutsideRTHB \n\x1e_equityWithLoanAfterOutsideRTHB\x10\n\x0e_suggestedSizeB\x0f\n\r_rejectReasonB\x0e\n\x0c_warningTextB\x10\n\x0e_completedTimeB\x12\n\x10_completedStatusB:\n\x16\x63om.ib.client.protobufB\x0fOrderStateProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'OrderState_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\017OrderStateProto\252\002\016IBApi.protobuf'
  _globals['_ORDERSTATE']._serialized_start=54
  _globals['_ORDERSTATE']._serialized_end=1835
# @@protoc_insertion_point(module_scope)



"""
Copyright (C) 2023 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi.object_implem import Object


class FaAllocationSamples(Object):
    #! [faupdatedgroup]
    FaUpdatedGroup = "".join(("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" ,
        "<ListOfGroups>" ,
            "<Group>" ,
                "<name>MyTestProfile1</name>" ,
                "<defaultMethod>ContractsOrShares</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                        "<amount>100.0</amount>" ,
                        "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                        "<amount>200.0</amount>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
                "<name>MyTestProfile2</name>" ,
                "<defaultMethod>Ratio</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                        "<amount>1.0</amount>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                        "<amount>2.0</amount>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
                "<name>MyTestProfile3</name>" ,
                "<defaultMethod>Percent</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                        "<amount>60.0</amount>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                        "<amount>40.0</amount>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
            "<name>MyTestProfile4</name>" ,
                "<defaultMethod>MonetaryAmount</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                        "<amount>1000.0</amount>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                        "<amount>2000.0</amount>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
                "<name>Group_1</name>" ,
                "<defaultMethod>NetLiq</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
                "<name>MyTestGroup1</name>" ,
                "<defaultMethod>AvailableEquity</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
                "<name>MyTestGroup2</name>" ,
                "<defaultMethod>NetLiq</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
                "<name>MyTestGroup3</name>" ,
                "<defaultMethod>Equal</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
            "<Group>" ,
                "<name>Group_2</name>" ,
                "<defaultMethod>AvailableEquity</defaultMethod>" ,
                "<ListOfAccts varName=\"list\">" ,
                    "<Account>" ,
                        "<acct>DU6202167</acct>" ,
                    "</Account>" ,
                    "<Account>" ,
                        "<acct>DU6202168</acct>" ,
                    "</Account>" ,
                "</ListOfAccts>" ,
            "</Group>" ,
        "</ListOfGroups>"))
    #! [faupdatedgroup]

def Test():
    print(FaAllocationSamples.FaUpdatedGroup)

if "__main__" == __name__:
    Test()





# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ExecutionDetails.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'ExecutionDetails.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ibapi.protobuf.Contract_pb2 as Contract__pb2
import ibapi.protobuf.Execution_pb2 as Execution__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x45xecutionDetails.proto\x12\x08protobuf\x1a\x0e\x43ontract.proto\x1a\x0f\x45xecution.proto\"\xa3\x01\n\x10\x45xecutionDetails\x12\x12\n\x05reqId\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12)\n\x08\x63ontract\x18\x02 \x01(\x0b\x32\x12.protobuf.ContractH\x01\x88\x01\x01\x12+\n\texecution\x18\x03 \x01(\x0b\x32\x13.protobuf.ExecutionH\x02\x88\x01\x01\x42\x08\n\x06_reqIdB\x0b\n\t_contractB\x0c\n\n_executionB@\n\x16\x63om.ib.client.protobufB\x15\x45xecutionDetailsProto\xaa\x02\x0eIBApi.protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ExecutionDetails_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ib.client.protobufB\025ExecutionDetailsProto\252\002\016IBApi.protobuf'
  _globals['_EXECUTIONDETAILS']._serialized_start=70
  _globals['_EXECUTIONDETAILS']._serialized_end=233
# @@protoc_insertion_point(module_scope)



"""
Copyright (C) 2024 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi.object_implem import Object
from ibapi.utils import intMaxString
from ibapi.utils import floatMaxString


class CommissionAndFeesReport(Object):
    def __init__(self):
        self.execId = ""
        self.commissionAndFees = 0.0
        self.currency = ""
        self.realizedPNL = 0.0
        self.yield_ = 0.0
        self.yieldRedemptionDate = 0  # YYYYMMDD format

    def __str__(self):
        return (
            "ExecId: %s, CommissionAndFees: %s, Currency: %s, RealizedPnL: %s, Yield: %s, YieldRedemptionDate: %s"
            % (
                self.execId,
                floatMaxString(self.commissionAndFees),
                self.currency,
                floatMaxString(self.realizedPNL),
                floatMaxString(self.yield_),
                intMaxString(self.yieldRedemptionDate),
            )
        )
